/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.2 Baustellensimulation
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.simbaustelle.prognose;

import java.text.MessageFormat;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.concurrent.TimeoutException;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.iav.gllib.gllib.domain.ApproximationTyp;
import de.bsvrz.iav.gllib.gllib.domain.EreignisTyp;
import de.bsvrz.iav.gllib.gllib.domain.MessQuerschnitt;
import de.bsvrz.iav.gllib.gllib.domain.MqGanglinie;
import de.bsvrz.iav.gllib.gllib.domain.PrognoseIntervall;
import de.bsvrz.iav.gllib.gllib.domain.comm.GanglinienPrognoseClient;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlProgAntwort;
import de.bsvrz.iav.gllib.gllib.domain.comm.PrognoseAnfrage;
import de.bsvrz.ibv.simbaustelle.domain.BerechnungsZeitRaum;
import de.bsvrz.ibv.simbaustelle.migration.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.ibv.simbaustelle.migration.util.UndefiniertException;
import de.bsvrz.ibv.simbaustelle.simbaustelle.BaustellenSimulation;
import de.bsvrz.ibv.simbaustelle.verwaltung.BaustellenSchnittstelle;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * eine Klasse zur Ermittlung des zufließenden Verkehrs zu einer Baustelle.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ZufliessenderVerkehr implements ZufliessenderVerkehrSchnittstelle {

    /**
     * der Logger für Debugausgaben.
     */
    private static final Debug LOGGER = Debug.getLogger();

    /**
     * die Ganglinie des Messquerschnitts unmittelbar vor der Baustelle.
     */
    private MqGanglinie gangLinie;

    /**
     * der vor der Baustelle liegende Messquerschnitt.
     */
    private final MessQuerschnittAllgemein mq;

    /**
     * erzeugt eine Klasse zur Bereitstellung des zur Baustelle zufließenden
     * Verkehrs.Der zufließende Verkehr ergibt sich aus der Ganglinie des
     * unmittelbar vor der Baustelle liegenden Messquerschnitts.
     *
     * @param applikation   die Applikation zur Baustellensimulation
     * @param baustelle     die Baustelle für die der zufließende Verkehr ermittelt
     *                      werden soll.
     * @param berechnungsMq optional vorgegebener Mq vor der Baustelle.
     * @param zeitRaum      der Zeitraum, für die eine Ermittlung erfolgen soll und
     *                      damit eine Ganglinie bestimmt werden muß
     * @throws UndefiniertException es konnte kein geeigneter Messquerschnitt oder
     *                              keine Ganglinie ermittelt werden.
     */
    public ZufliessenderVerkehr(BaustellenSimulation applikation, final BaustellenSchnittstelle baustelle,
            final MessQuerschnittAllgemein berechnungsMq, final BerechnungsZeitRaum zeitRaum)
            throws UndefiniertException {

        if (berechnungsMq != null) {
            mq = berechnungsMq;
        } else {
            mq = baustelle.getMessquerschnittDavor();
        }

        if (mq == null) {
            throw new UndefiniertException(
                    "Der Messquerschnitt zur Bestimmung des Zuflusses konnte nicht ermittelt werden!");
        }

        final PrognoseAnfrage anfrage = new PrognoseAnfrage(MessQuerschnitt.of(mq.getPid()),
                PrognoseIntervall.of(zeitRaum.getStartZeitpunkt(), zeitRaum.getEndeZeitpunkt()), true);

        final SystemObject baustellenEreignisTyp = applikation.getVerbindung().getDataModel()
                .getObject(BaustellenSimulation.getBstErgTypPid());
        if (baustellenEreignisTyp != null) {
            anfrage.getEreignisTypen().add(EreignisTyp.of(baustellenEreignisTyp.getPid()));
        }

        LOGGER.fine("Anfrage an Ganglinienprognose versenden");
        GanglinienPrognoseClient prognose = applikation.getGanglinienprognose();
        try {
            GlProgAntwort antwort = prognose.sendeAnfrage(Collections.singletonList(anfrage), Duration.ofMinutes(2));
            verarbeiteAntwort(antwort);
        } catch (final TimeoutException e) {
            String message = MessageFormat.format(
                    "Ganglinienabfrage für Baustelle {0} im Intervall {1} ist fehlgeschlagen: {2}", baustelle, zeitRaum,
                    e.getMessage());
            applikation.getBetriebsMeldungsSender().warnung(baustelle.getObjekt().getPid(), message);
            LOGGER.warning(message);
        }
    }

    private void verarbeiteAntwort(GlProgAntwort antwort) {

        LOGGER.fine("Verarbeite Antwort von Ganglinienprognose");
        for (final MqGanglinie gl : antwort.getGanglinien()) {
            final MessQuerschnitt glMq = gl.getMessQuerschnitt();
            if (glMq == null) {
                continue;
            }
            if (glMq.getPid().equals(mq.getPid())) {
                gangLinie = gl;
                if (BaustellenSimulation.isPolyLinie()) {
                    gangLinie.setApproximationDaK(ApproximationTyp.POLYLINE);
                }
            }
        }

        synchronized (this) {
            this.notifyAll();
        }
    }

    @Override
    public final long getFzg(final LocalDateTime zs) throws UndefiniertException {

        if (gangLinie == null) {
            throw new UndefiniertException(
                    "Zufliessender Verkehr: Es konnte keine Ganglinie für MQ:" + mq + " ermittelt werden");
        }
        final Double wert = gangLinie.getStuetzstelle(zs).getWert().getQB();
        if (wert == null) {
            throw new UndefiniertException(
                    "In der Ganglinie des MQ: " + mq + " existiert kein Wert für den Zeitpunkt: " + zs);
        }
        return wert.longValue();
    }

    @Override
    public long getQPkw(LocalDateTime zs) {
        if (gangLinie == null) {
            LOGGER.warning("Zufliessender Verkehr: Es konnte keine Anzahl der Pkw für MQ:" + mq + " ermittelt werden");
            return -1;
        }
        final Double wert = gangLinie.getStuetzstelle(zs).getWert().getQPkw();
        if (wert == null) {
            LOGGER.warning("In der Ganglinie des MQ: " + mq + " existiert kein Wert für den Zeitpunkt: "
                    + zs.format(DateTimeFormatter.ISO_LOCAL_DATE_TIME));
            return -1;
        }
        return wert.longValue();
    }

    @Override
    public long getQLkw(LocalDateTime zs) {
        if (gangLinie == null) {
            LOGGER.warning("Zufliessender Verkehr: Es konnte keine Anzahl der Lkw für MQ:" + mq + " ermittelt werden");
            return -1;
        }
        final Double wert = gangLinie.getStuetzstelle(zs).getWert().getQLkw();
        if (wert == null) {
            LOGGER.warning("In der Ganglinie des MQ: " + mq + " existiert kein Wert für den Zeitpunkt: "
                    + (zs.format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)));
            return -1;
        }
        return wert.longValue();
    }
}
