/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.2 Baustellensimulation
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.simbaustelle.verwaltung;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.iav.gllib.gllib.shared.LocalDateTimeKonverter;
import de.bsvrz.ibv.simbaustelle.migration.modell.ObjektFactory;
import de.bsvrz.ibv.simbaustelle.migration.modell.SystemObjekt;
import de.bsvrz.ibv.simbaustelle.migration.modell.verkehr.objekte.StrassenSegment;
import de.bsvrz.ibv.simbaustelle.migration.util.UndefiniertException;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Daten einer Mdm-Baustelle.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class MdmBaustellenDaten implements BaustellenDatenSchnittstelle, ClientReceiverInterface {

	private static final Debug LOGGER = Debug.getLogger();
	private ClientDavInterface connection;
	private MdmBaustelle mdmBaustelle;
	private DataDescription parameterDesc;

	private LocalDateTime start;
	private LocalDateTime ende;
	private long endOffset;
	private final List<StrassenSegment> segmente = new ArrayList<>();

	public MdmBaustellenDaten(ClientDavInterface connection, MdmBaustelle mdmBaustelle) {
		this.connection = connection;
		this.mdmBaustelle = mdmBaustelle;

		AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.mdmBaustellenDatenGesamt");
		Aspect asp = connection.getDataModel().getAspect("asp.parameterSoll");
		parameterDesc = new DataDescription(atg, asp);

		connection.subscribeReceiver(this, mdmBaustelle.getObjekt(), parameterDesc, ReceiveOptions.normal(),
				ReceiverRole.receiver());
	}

	@Override
	public LocalDateTime getStart() throws UndefiniertException {
		return start;
	}

	@Override
	public LocalDateTime getEnde() throws UndefiniertException {
		return ende;
	}

	@Override
	public long getRestKapazitaet(LocalDateTime zs) throws UndefiniertException {
		throw new UndefiniertException("Restkapazität für MDM-Baustellen wurde nicht mit dem Simulationsauftrag gesetzt");
	}

	@Override
	public long getEndOffset() throws UndefiniertException {
		return endOffset;
	}

	@Override
	public void update(ResultData[] results) {
		for (ResultData result : results) {
			if (!result.getDataDescription().getAttributeGroup().equals(parameterDesc.getAttributeGroup())) {
				continue;
			}
			updateParameter(result.getData());
		}
	}

	private void updateParameter(Data data) {

		if (data == null) {
			resetParameter();
			return;
		}

		start = LocalDateTimeKonverter.fromMillis(data.getItem("GesamtMassnahme").getTimeValue("GmBeginn").getMillis());
		ende = LocalDateTimeKonverter.fromMillis(data.getItem("GesamtMassnahme").getTimeValue("GmEnde").getMillis());

		Array ortsRefVrzArray = data.getItem("GesamtMassnahme").getItem("GmOrtsReferenzen").getArray("OrtsRefVrz");
		if (ortsRefVrzArray.getLength() > 0) {
			Data ortsRefVrz = ortsRefVrzArray.getItem(0);
			endOffset = ortsRefVrz.getScaledValue("VonOffset").longValue();

			Array segmentArray = ortsRefVrz.getArray("ListeSegmente");
			for (int idx = 0; idx < segmentArray.getLength(); idx++) {
				String segmentPid = segmentArray.getTextValue(idx).getText();
				if (segmentPid != null) {
					SystemObjekt modellobjekt = ObjektFactory.getInstanz().getModellobjekt(segmentPid);
					if (modellobjekt instanceof StrassenSegment) {
						segmente.add((StrassenSegment) modellobjekt);
					} else {
						LOGGER.warning(
								"Das ReferenzObjekt \"" + segmentPid + "\" beschreibt kein Straßensegment!");
					}
				}
			}

		} else {
			endOffset = 0;
		}
	}

	private void resetParameter() {
		start = LocalDateTime.MIN;
		ende = LocalDateTime.MIN;
		endOffset = 0;
		segmente.clear();
	}

	@Override
	public void disconnect() {
		connection.unsubscribeReceiver(this, mdmBaustelle.getObjekt(), parameterDesc);
	}

	@Override
	public List<StrassenSegment> getSegmente() throws UndefiniertException {
		if (segmente.isEmpty()) {
			throw new UndefiniertException("Die Strassensegmente der Baustelle konnten nicht ermittelt werden!");
		}
		return segmente;
	}

}
